from .util import warning

def read_hex_timestamp(hexText):
    return int(hexText,16) / (1000 * 1000.0)

class Message:
    def calculate_duration(self):
        offset_bits=1+11+1+6+15+1+1+1 # fixed protocol overhead till ack slot
        packet_length_bits=offset_bits+self.payload_length*8
        max_stuffed_packet_length_bits=packet_length_bits*7/6
        packet_duration_seconds=max_stuffed_packet_length_bits*1.0/self.baudrate
        return packet_duration_seconds

    def calculate_inter_frame_gap(self):
        gap_bits=7+3
        return gap_bits*1.0/self.baudrate
    
    def to_string(self):
        payload_string='[' + ' '.join( ["%02X" % x for x in self.payload ] ) + ']'
        return "%06d %02X %0.6f %04X %02X %s " % (self.counter, self.channel, self.timestamp, self.message_id, self.payload_length, payload_string)
    pass

def create_message_from_data(dataString, message_counter, baudrate_config_dict):
    m=Message()
    m.counter=message_counter
    (m.channel,hex_timestamp, m.message_id, m.payload_length, payload_string)=dataString.split(',',4)
    m.channel=int(m.channel,16)
    m.message_id=int(m.message_id,16)
    m.payload_length=int(m.payload_length,16)
    m.timestamp=read_hex_timestamp(hex_timestamp)

    baudrate_key="CH%dBAUD" % (m.channel)
    m.baudrate=baudrate_config_dict[baudrate_key]
        
    if payload_string[-1]==',':
        payload_string=payload_string[:-1]        
    m.payload=[int(x,16) for x in payload_string.split(',')]

    return m

def parse_message_file(filename, line_limit=None):
    """ reads the schnueffel messages from the given file and returns a tuple (list() of message objects, configuration dictionary) """
    # a transmission constist of 1x packet number, 0-nx data, 1x ts of packet creation
    min_timestamp = None
    last_packetnumber = None
    max_timestamp = None

    sys_max_timestamp=(2^32-1) / (1000 * 1000.0) # largest possible timestamp due to integer limitations

    messages=[]
    message_counter=0
    line_num = 0
    preamble_seen = False
    config_dict=dict()
    with open(filename,"r") as f:
        for line in f:
            line=line.strip()
            # print("Line is " + line)
            line_num +=1


            if line.startswith("#"):
                (key,value)=line[1:].split(":")

                if key=="TS":
                    if preamble_seen: 
                        timestamp=read_hex_timestamp(value)
                        if max_timestamp:
                            if timestamp < max_timestamp:
                                warning("Line %d: Time ran backwards (packet): %f => %f" % (line_num, max_timestamp,timestamp))
                        max_timestamp=timestamp                

                    # print "TS:", timestamp
                elif key=="PN":
                    if preamble_seen: 
                        packetnumber=int(value,16)
                        # print "PN:", packetnumber
                        if last_packetnumber:
                            if packetnumber != last_packetnumber+1:
                                if packetnumber == 0 and last_packet_number ==  2**32-1:
                                    pass
                                else:
                                    warning("Line %d: Packet number did not increase by one: %d => %d ( %d packets lost )" % (line_num, last_packetnumber,packetnumber,packetnumber-last_packetnumber-1))

                        last_packetnumber=packetnumber
                elif key=="Version":
                    print "Schnueffel Version %s" % (value)
                    preamble_seen = True
                    config_dict[key]=value
                elif key in ("CH1BAUD","CH2BAUD"):
                    config_dict[key]=int(value,16)*1000 # convert kilobaud to baud
                else:
                    warning("Line %d: Unknown keyword %s" % key )
                    config_dict[key]=value
            else: # no command, must be data
                # print "DATA"
                if preamble_seen: 
                    m=create_message_from_data(line, message_counter, config_dict)
                    message_counter+=1
                    # print m.to_string()
                    messages.append(m)
                    if max_timestamp:
                        if m.timestamp < max_timestamp:
                            if  max_timestamp > sys_max_timestamp - 0.1 and m.timestamp < 0.1:
                                pass
                            else:
                                warning("Time ran backwards (message): %d => %d" % (max_timestamp,timestamp))
                    max_timestamp=m.timestamp

            if line_limit and line_num >= line_limit:
                break
    return messages, config_dict
            
