/* Includes ------------------------------------------------------------------*/

#include "main.h"
#include "stm32f4xx_can.h"
#include "can_example.h"
/** @addtogroup STM32F4xx_StdPeriph_Examples
  * @{
  */

/** @addtogroup CAN_LoopBack
  * @{
  */ 

/* Private define ------------------------------------------------------------*/
/* Private typedef -----------------------------------------------------------*/


/* Private macro -------------------------------------------------------------*/

/* Private variables ---------------------------------------------------------*/


/* Private function prototypes -----------------------------------------------*/


/* Global functions ---------------------------------------------------------*/

/**
  * @brief  Initialize Can Module and Pins
  * @param  None
  * @retval None
  */
void CAN_Initialisierung()
{
	  CAN_InitTypeDef        CAN_InitStructure;
	  CAN_FilterInitTypeDef  CAN_FilterInitStructure;
	  GPIO_InitTypeDef GPIO_InitStructure;
	  /* CAN GPIOs configuration **************************************************/

	  /* Enable GPIO clock */
	  RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_GPIOD, ENABLE);
	  RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_GPIOB, ENABLE);

	  /* Connect CAN pins to AF9*/
	  GPIO_PinAFConfig(GPIOD, GPIO_PinSource0, GPIO_AF_CAN1);
	  GPIO_PinAFConfig(GPIOD, GPIO_PinSource1, GPIO_AF_CAN1);

	  /* Configure CAN RX and TX pins*/
	  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_1 | GPIO_Pin_0;
	  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF;
	  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_100MHz;
	  GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
	  GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_UP;
	  GPIO_Init(GPIOD, &GPIO_InitStructure);

	  /* Connect CAN pins to AF9*/
	  GPIO_PinAFConfig(GPIOB, GPIO_PinSource5, GPIO_AF_CAN2);
	  GPIO_PinAFConfig(GPIOB, GPIO_PinSource6, GPIO_AF_CAN2);

	  /* Configure CAN RX and TX pins*/
	  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_6 | GPIO_Pin_5;
	  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF;
	  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_100MHz;
	  GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
	  GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_UP;
	  GPIO_Init(GPIOB, &GPIO_InitStructure);


	  /* CAN register init */
	  CAN_DeInit(CAN1);
	  CAN_DeInit(CAN2);

	  // RCC_APB1PeriphClockCmd(RCC_APB1Periph_CAN1, ENABLE);
	  RCC_APB1PeriphClockCmd(RCC_APB1Periph_CAN1 | RCC_APB1Periph_CAN2, ENABLE);
	  /* CAN cell init */
	  CAN_InitStructure.CAN_TTCM = DISABLE;
	  CAN_InitStructure.CAN_ABOM = DISABLE;
	  CAN_InitStructure.CAN_AWUM = DISABLE;
	  CAN_InitStructure.CAN_NART = ENABLE;
	  CAN_InitStructure.CAN_RFLM = DISABLE;
	  CAN_InitStructure.CAN_TXFP = DISABLE;
	  CAN_InitStructure.CAN_Mode = CAN_Mode_Normal;
	  CAN_InitStructure.CAN_SJW = CAN_SJW_2tq;	//<- Hier die max sprung Weite zur Taktkorrektur einstellen

	  /*Mit CAN_Prescaler kann eingestellt werden wie breit die Time Quanten werden sollen
	   *Mit CAN_BS1 und CAN_BS2 werden die Quanten aus die Segmente verteilt die gesammt
	   *Anzahl ergibt sich aus BS1+BS2+1 was dann die Bitbreite und somit die Bitrate bestimmt
	   *Der sample point befindet sich am ende von CAN_BS1
	   *Die maximale Nachsynchronisations Sprungweite kann mit CAN_SJW eingestellt werden

	   *CAN Baudrate = 500kbps (CAN clocked at 42 MHz)
	   *Bitrate CAN1 																			*/
	  CAN_InitStructure.CAN_BS1 = CAN_BS1_16tq;//7
	  CAN_InitStructure.CAN_BS2 = CAN_BS2_4tq;//2
	  CAN_InitStructure.CAN_Prescaler = 4;//210;
	  CAN_Init(CAN1, &CAN_InitStructure);

	  /* CAN2  */
	  CAN_InitStructure.CAN_SJW = CAN_SJW_2tq;	//<- Hier die max sprung Weite zur Taktkorrektur einstellen

	  /* CAN Baudrate = 50kbps (CAN clocked at 42 MHz) */
	  /* Bitrate CAN2 */
	  CAN_InitStructure.CAN_BS1 = CAN_BS1_16tq;//7
	  CAN_InitStructure.CAN_BS2 = CAN_BS2_4tq;//2
	  CAN_InitStructure.CAN_Prescaler = 40;//210;	//<- Hier die Breite der Quanten ver�ndern

	  CAN_Init(CAN2, &CAN_InitStructure);


	  /* CAN filter init */
	  CAN_FilterInitStructure.CAN_FilterNumber = 0;

	  CAN_FilterInitStructure.CAN_FilterMode = CAN_FilterMode_IdMask;
	  CAN_FilterInitStructure.CAN_FilterScale = CAN_FilterScale_32bit;
	  CAN_FilterInitStructure.CAN_FilterIdHigh = 0x0000;
	  CAN_FilterInitStructure.CAN_FilterIdLow = 0x0000;
	  CAN_FilterInitStructure.CAN_FilterMaskIdHigh = 0x0000;
	  CAN_FilterInitStructure.CAN_FilterMaskIdLow = 0x0000;
	  CAN_FilterInitStructure.CAN_FilterFIFOAssignment = CAN_FIFO1;

	  CAN_FilterInitStructure.CAN_FilterActivation = ENABLE;
	  CAN_FilterInit(&CAN_FilterInitStructure);


	  CAN_FilterInitStructure.CAN_FilterNumber = 14;

	  CAN_FilterInitStructure.CAN_FilterMode = CAN_FilterMode_IdMask;
	  CAN_FilterInitStructure.CAN_FilterScale = CAN_FilterScale_32bit;
	  CAN_FilterInitStructure.CAN_FilterIdHigh = 0x0000;
	  CAN_FilterInitStructure.CAN_FilterIdLow = 0x0000;
	  CAN_FilterInitStructure.CAN_FilterMaskIdHigh = 0x0000;
	  CAN_FilterInitStructure.CAN_FilterMaskIdLow = 0x0000;
	  CAN_FilterInitStructure.CAN_FilterFIFOAssignment = CAN_FIFO0;

	  CAN_FilterInitStructure.CAN_FilterActivation = ENABLE;
	  CAN_FilterInit(&CAN_FilterInitStructure);


}

/**
  * @brief  CAN transmit
  * @param  None
  * @retval None
  */
void CAN_Send(CAN_TypeDef *CAN,CanTxMsg *Message)
{
	volatile uint8_t TransmitMailbox = 0;

	TransmitMailbox = CAN_Transmit(CAN, Message);
}
/**
  * @}
  */ 

/******************* (C) COPYRIGHT 2011 STMicroelectronics *****END OF FILE****/
