/**
  ******************************************************************************
  * @file    main.c
  * @author  MCD Application Team
  * @version V1.0.0
  * @date    31-October-2011
  * @brief   Main program body
  ******************************************************************************
  * @attention
  *
  * THE PRESENT FIRMWARE WHICH IS FOR GUIDANCE ONLY AIMS AT PROVIDING CUSTOMERS
  * WITH CODING INFORMATION REGARDING THEIR PRODUCTS IN ORDER FOR THEM TO SAVE
  * TIME. AS A RESULT, STMICROELECTRONICS SHALL NOT BE HELD LIABLE FOR ANY
  * DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING
  * FROM THE CONTENT OF SUCH FIRMWARE AND/OR THE USE MADE BY CUSTOMERS OF THE
  * CODING INFORMATION CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
  *
  * <h2><center>&copy; Portions COPYRIGHT 2011 STMicroelectronics</center></h2>
  ******************************************************************************
  */

/**
  ******************************************************************************
  * <h2><center>&copy; Portions COPYRIGHT 2012 Embest Tech. Co., Ltd.</center></h2>
  * @file    main.c
  * @author  CMP Team
  * @version V1.0.0
  * @date    28-December-2012
  * @brief   Main program body      
  *          Modified to support the STM32F4DISCOVERY, STM32F4DIS-BB and
  *          STM32F4DIS-LCD modules. 
  ******************************************************************************
  * @attention
  *
  * THE PRESENT FIRMWARE WHICH IS FOR GUIDANCE ONLY AIMS AT PROVIDING CUSTOMERS
  * WITH CODING INFORMATION REGARDING THEIR PRODUCTS IN ORDER FOR THEM TO SAVE
  * TIME. AS A RESULT, Embest SHALL NOT BE HELD LIABLE FOR ANY DIRECT, INDIRECT
  * OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING FROM THE CONTENT
  * OF SUCH FIRMWARE AND/OR THE USE MADE BY CUSTOMERS OF THE CODING INFORMATION
  * CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
  ******************************************************************************
  */ 
/* Includes ------------------------------------------------------------------*/
#include <string.h>
#include "stm32f4x7_eth.h"
#include "netconf.h"
#include "main.h"
#include "udp_echoserver.h"
#include "serial_debug.h"
#include "stm32f4xx_gpio.h"
#include "stm32f4xx_rcc.h"
#include "stm32f4xx_syscfg.h"
#include "stm32f4xx_tim.h"
#include "stm32f4xx_can.h"
#include "misc.h"
#include "lwip/pbuf.h"
#include "lwip/udp.h"
#include "can_example.h"
// #include "svn_version.h"
#define SVN_REVISION "1000"

/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/


/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
uint32_t LocalTime;

static uint8_t buffer1[1400];
static uint8_t buffer2[1400];
static uint8_t firstpacket[100];

/* Global variables ---------------------------------------------------------*/
uint8_t *p_buf_can=buffer1;
uint16_t can_work=13;
uint8_t *p_buf_udp=buffer2;

/* Private function prototypes -----------------------------------------------*/
void LED_Init(void);
void wait(uint32_t waittime);
void InterruptCAN_Init(void);
/* Private functions ---------------------------------------------------------*/

/**
  * @brief  Main program.
  * @param  None
  * @retval None
  */
int main(void)
{
	TIM_TimeBaseInitTypeDef  TIM_TimeBaseInitStructure;
	struct ip_addr adress;
	struct udp_pcb *upcb;
	int8_t err,firsttime=0;
	struct pbuf *p;
	static uint32_t packettimestamp,packetId=0;
	uint8_t *tempbufferpointer;
	uint8_t starttext[]="#PN:";
	uint8_t endtext[]="#TS:";
	uint8_t baud1text[]="#CH1BAUD:";
	uint8_t baud2text[]="#CH2BAUD:";
	uint8_t version[]="#Version:";
	uint16_t bufferlength=0;
// 	uint32_t versionnumber=SVN_REVISION;
	uint16_t workcountconfig=0;
	uint32_t baud;

	//Initialize the system use external 8MHz crystal
	SystemInit();

	/*!< At this stage the microcontroller clock setting is already configured to
	168 MHz, this is done through SystemInit() function which is called from
	startup file (startup_stm32f4xx.s) before to branch to application main.
	To reconfigure the default setting of SystemInit() function, refer to
	system_stm32f4xx.c file
	*/

	/*Initialize LCD and Leds */
	LED_Init();


	//TIM_DeInit(TIM2);
	/* TIM2 clock enable */
	RCC_APB1PeriphClockCmd(RCC_APB1Periph_TIM2, ENABLE);

	TIM_TimeBaseStructInit(&TIM_TimeBaseInitStructure);
	//initialize the Timer 2 in counter up mode and 1MHz frequency
	//Timer2 is a 32bit timer
	TIM_TimeBaseInitStructure.TIM_CounterMode=TIM_CounterMode_Up;
	TIM_TimeBaseInitStructure.TIM_Period=0xFFFFFFFF;
	TIM_TimeBaseInitStructure.TIM_Prescaler=84;
	TIM_TimeBaseInit(TIM2,&TIM_TimeBaseInitStructure);

	TIM_CounterModeConfig(TIM2,TIM_CounterMode_Up);
	//Start Timer2
	TIM_Cmd(TIM2, ENABLE);


	/* configure ethernet */
	ETH_BSP_Config();

	/* Initialize the LwIP stack */
	LwIP_Init();

	/* UDP echoserver */
	udp_echoserver_init();

	/* Create a new UDP control block  */
	upcb = udp_new();

	/* Set up the target PC IP  */
	IP4_ADDR(&adress,IP_PC_ADDR0,IP_PC_ADDR1,IP_PC_ADDR2,IP_PC_ADDR3);

	/*Configure the Interrupt*/
	InterruptCAN_Init();
	/*Configure the CAN IF*/
	CAN_Initialisierung();

	/*Enable CAN Interrupt*/
	CAN_ITConfig(CAN2, CAN_IT_FMP0, ENABLE);
	/*Enable CAN Interrupt*/
	CAN_ITConfig(CAN1, CAN_IT_FMP1, ENABLE);

	/* Infinite loop */
	while (1)
	{

	  /* check if any packet received */
	if (ETH_CheckFrameReceived())
		{
		  /* process received ethernet packet */
		  LwIP_Pkt_Handle();
		}


	 /* handle periodic timers for LwIP *///wird fr den ARP timer verwendet
	Time_Update();
	LwIP_Periodic_Handle(LocalTime);
	//Get the timestamp for the UDP packet
	packettimestamp=TIM_GetCounter(TIM2);

	//Send once an packet with the version number and the Baudrate
	if(firsttime==0)
		{
			//write the start sequence into the buffer

			memcpy (firstpacket,starttext, sizeof(starttext));

			workcountconfig=sizeof(starttext)-1;

			//write the packet Id
			Translate32toAscii(packetId++,&firstpacket[4]);

			workcountconfig+=8;

			firstpacket[11]=0x0a;

			workcountconfig++;
			//Start sequence finished
			//-----------------------------------------------------------------
			memcpy (&firstpacket[workcountconfig-1],version, sizeof(version));

			workcountconfig+=sizeof(version);

			memcpy(&firstpacket[workcountconfig-2],SVN_REVISION,sizeof(SVN_REVISION));

			workcountconfig+=sizeof(SVN_REVISION)-3;

			firstpacket[workcountconfig]=0x0a;

			workcountconfig++;

			memcpy (&firstpacket[workcountconfig],baud1text, sizeof(baud1text));

			workcountconfig+=sizeof(baud1text)-1;

			baud=CH1BAUD;

			Translate32toAscii(baud,&firstpacket[workcountconfig]);

			workcountconfig+=8;

			firstpacket[workcountconfig]=0x0a;

			workcountconfig++;

			memcpy (&firstpacket[workcountconfig],baud2text, sizeof(baud2text));

			workcountconfig+=sizeof(baud2text)-1;

			baud=CH2BAUD;

			Translate32toAscii(baud,&firstpacket[workcountconfig]);

			workcountconfig+=8;


			//-----------------------------------------------------------------

			firstpacket[workcountconfig]=0x0a;

			workcountconfig++;

			memcpy (&firstpacket[workcountconfig],endtext, sizeof(endtext));

			workcountconfig+=sizeof(endtext)-1;

			//write the packet Timestamp
			Translate32toAscii(packettimestamp,&firstpacket[workcountconfig]);

			workcountconfig+=8;

			firstpacket[workcountconfig]=0xa;

			workcountconfig++;

			//End sequence finished

			//get memory for the pbuf
			p=pbuf_alloc(PBUF_TRANSPORT, workcountconfig, PBUF_REF);

			//set the payload to the used buffer
			p->payload=firstpacket;


			//GPIO_SetBits(GPIOD, GPIO_Pin_13);
			err=udp_sendto(upcb, p, &adress, DEST_PORT);
			firsttime=1;
		}

	wait(2500);



	//-----------------------------------------------------------------------------
	/*DISABLE CAN Interrupt*/
	CAN_ITConfig(CAN2, CAN_IT_FMP0, DISABLE);
	/*DISABLE CAN Interrupt*/
	CAN_ITConfig(CAN1, CAN_IT_FMP1, DISABLE);

	//switch the pointers
	tempbufferpointer=p_buf_can;
	p_buf_can=p_buf_udp;
	p_buf_udp=tempbufferpointer;

	//save the can working index
	bufferlength=can_work;

	//reset the can working index
	can_work=13;

	//Get the timestamp for the UDP packet
	packettimestamp=TIM_GetCounter(TIM2);

	/*Enable CAN Interrupt*/
	CAN_ITConfig(CAN2, CAN_IT_FMP0, ENABLE);
	/*Enable CAN Interrupt*/
	CAN_ITConfig(CAN1, CAN_IT_FMP1, ENABLE);
	//------------------------------------------------------------------------------

	//write the start sequence into the buffer

	memcpy (p_buf_udp,starttext, sizeof(starttext));

	//write the packet Id
	Translate32toAscii(packetId++,&p_buf_udp[4]);

	p_buf_udp[12]=0x0a;
	//Start sequence finished

	memcpy (&p_buf_udp[bufferlength],endtext, sizeof(endtext));

	//write the packet Timestamp
	Translate32toAscii(packettimestamp,&p_buf_udp[bufferlength+4]);

	p_buf_udp[bufferlength+12]=0xa;
	//End sequence finished

	//get memory for the pbuf
	p=pbuf_alloc(PBUF_TRANSPORT, bufferlength+13, PBUF_REF);

	//set the payload to the used buffer
	p->payload=p_buf_udp;


	//GPIO_SetBits(GPIOD, GPIO_Pin_13);
	err=udp_sendto(upcb, p, &adress, DEST_PORT);

	GPIO_SetBits(GPIOD, GPIO_Pin_12);

	if(err!=0)
		while(1)
			{
				GPIO_SetBits(GPIOD, GPIO_Pin_14);
				GPIO_ResetBits(GPIOD, GPIO_Pin_12);
			}


	/* Free the p buffer */
	pbuf_free(p);

	}


}

/**
  * @brief  Updates the system local time
  * @param  None
  * @retval None
  */
void Time_Update(void)
{
	static uint32_t timeold,time;

	if(time>timeold)
	{
		time=TIM_GetCounter(TIM2)/1000-timeold;
		LocalTime += time;
	}

  timeold=time;

}

/**
  * @brief  Inserts a wait time.
  * @param  nCount: number of s to wait for.
  * @retval None
  */
void wait(uint32_t waittime)
{
	static uint32_t timestart,timediff;

	//get the time
	timestart=TIM_GetCounter(TIM2);

	timediff=0;

	while(timediff<waittime)
		timediff=TIM_GetCounter(TIM2)-timestart;

	//get the time
	timestart=TIM_GetCounter(TIM2);
}

/**
  * @brief  Initializes the LEDs resources.
  * @param  None
  * @retval None
  */
void LED_Init(void)
{
	GPIO_InitTypeDef GPIO_InitStructure;

  /* Initialize Discovery's LEDs */
  RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_GPIOD, ENABLE);

  /* Configure PD15 in output pushpull mode */
  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_12	|	GPIO_Pin_13	|  GPIO_Pin_14;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_OUT;
  GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
  GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL;
  GPIO_Init(GPIOD, &GPIO_InitStructure);


}

/**
  * @brief  translates uint8 to ascii signes and writes it to the assigned place
  * @param  val -> number to translate
  * @param  pAscii -> space to put the signs
  * @retval None
  */
void Translate8toAscii(uint8_t val, uint8_t *pAscii)
{
uint8_t temp;
int8_t i=0;
// uint8_t addvalue;


for(i=0;i<=1;i++)
{
	temp=val & 0xf0;
	pAscii[i]=(temp>>4)+0x30;

	if(pAscii[i]>0x39)
		pAscii[i]=(pAscii[i])+7;

	val=val<<4;

}


}
/**
  * @brief  translates uint32 to ascii signes and writes it to the assigned place
  * @param  val -> number to translate
  * @param  pAscii -> space to put the signs
  * @retval None
  */
void Translate32toAscii(uint32_t val, uint8_t *pAscii)
{
uint32_t temp;
int8_t i=0;
// uint8_t addvalue;


for(i=0;i<=7;i++)
{
	temp=val & 0xf0000000;
	pAscii[i]=(temp>>28)+0x30;

	if(pAscii[i]>0x39)
		pAscii[i]=(pAscii[i])+7;

	val=val<<4;

}


}

/**
  * @brief  Configures the NVIC.
  * @param  None
  * @retval None
  */
void InterruptCAN_Init(void)
{
  NVIC_InitTypeDef NVIC_InitStructure;

  NVIC_InitStructure.NVIC_IRQChannel = CAN2_RX0_IRQn;

  NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 0;
  NVIC_InitStructure.NVIC_IRQChannelSubPriority =191+5;
  NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;

  NVIC_Init(&NVIC_InitStructure);

  NVIC_InitStructure.NVIC_IRQChannel = CAN1_RX1_IRQn;

  NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 0;
  NVIC_InitStructure.NVIC_IRQChannelSubPriority =191+5;
  NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;

  NVIC_Init(&NVIC_InitStructure);
}
/*********** Portions COPYRIGHT 2012 Embest Tech. Co., Ltd.*****END OF FILE****/
